/* ******************* */
/* Fitxer sfTracking.c */
/* ******************* */

#include "saphira.h"
#include "sonar.h"
#include "sfTracking.h"
#include <assert.h>

// ---------------- 
// Funci principal 
// ---------------- 

void c_calcul_contorn(void)
{
	c_agafar_dades();
	c_aplicar_filtres();
	calcular_angle_cami();
	c_calcul_control(c_calcularXReg(),c_calcularYReg(),&angle_control,&distancia_control);
}

// -----------------------------
// Funcions que agafen les dades
// -----------------------------

void c_agafar_dades(void)
{
	int i;

	s_actualitzar();							//S'actualitza el sonar

	llEliminarTotsPunts(cami);					// Esborrem la llista cami
	llEliminarTotsPunts(llista);				// Esborrem la llista llista
	llEliminarTotsPunts(llista_emergencia);		// Esborrem la llista llista_emergencia

	// Insertem els punts del sonar

	i = 1;

	while (i<=s_elements())
	{
		llAfegirPunt(llista,s_x(i),s_y(i),0.0);
		llAfegirPunt(llista_emergencia,s_x(i),s_y(i),0.0);

		i++;
	}
}

// ----------------------------------------------
// Funcions que realitzen un filtrat de les dades
// ----------------------------------------------

void c_aplicar_filtres(void)
{
	if (filtre_angle_activat)
	{
		c_filtre_angle(llista,p_alfa_1,p_alfa_2,p_max_correcte);
		if (llNumPunts(llista)>0) c_filtre_angle(llista_emergencia,p_alfa_1,p_alfa_2,p_max_correcte);
	}

	if (filtre_focus_activat)
	{
		c_filtre_focus(llista,p_dist_focus);
		if (llNumPunts(llista)>0) c_filtre_focus(llista_emergencia,p_focus_control);
	}

	if (filtre_recta_activat)
	{
		c_filtre_recta(llista);
	}
}

void c_filtre_angle(LlistaPunts l, float alfa_1, float alfa_2, float max_correcte)
{
	int i;

	i = 0;
	while (i<llNumPunts(l))
	{
		if (!correcte_filtre_angle(llObtenirXPunt(l,i),llObtenirYPunt(l,i),alfa_1,alfa_2,max_correcte))
		{
			llEliminarPunt(l,i);
			continue;
		}

		i++;
	}
}

int correcte_filtre_angle(float x, float y, float alfa_1, float alfa_2, float max_correcte)
{
	int resultat;
	float dist_max;

	dist_max = distancia_a_mantenir*max_correcte; 

	resultat = 1;

	if (!seguir_dreta)
	{
		alfa_1 = -alfa_1;
		alfa_2 = -alfa_2;

		resultat = resultat&&((y-x*tan(alfa_1*2*PI/360))>=0);
		resultat = resultat&&((y-x*tan(alfa_2*2*PI/360))>=0);
	}
	else
	{
		alfa_1 = alfa_1;
		alfa_2 = alfa_2;

		resultat = resultat&&((y-x*tan(alfa_1*2*PI/360))<=0);
		resultat = resultat&&((y-x*tan(alfa_2*2*PI/360))<=0);
	}

	resultat = resultat&&(distancia(x,y)<=dist_max);

	return resultat;
}

void c_filtre_focus(LlistaPunts l, float dist_focus)
{
	int i, i_min;
	float x_min, y_min;

	if (llNumPunts(l)<1) return;

	i_min = MinimaDistanciaLlistaPunts(l);
	x_min = llObtenirXPunt(l,i_min);
	y_min = llObtenirYPunt(l,i_min);

	i = 0;
	while (i<llNumPunts(l))
	{
		if (!(distancia(x_min-llObtenirXPunt(l,i),y_min-llObtenirYPunt(l,i))<=dist_focus))
		{
			llEliminarPunt(l,i);
			continue;
		}

		i++;
	}
}

void c_filtre_recta(LlistaPunts l)
{
	// Posa es punts sobre una recta

	float x0,y0,vx,vy;
	float x_final,y_final;
	int i;

	if (llNumPunts(l)<2) return;

	c_calcul_recta_regressio(l,&x0,&y0,&vx,&vy,&r_reg);
	c_calcul_r(l,x0,y0,vx,vy,&r_reg);
	
	i = 0;
	while (i<llNumPunts(l))
	{
		c_calcul_projeccio(llObtenirXPunt(l,i),llObtenirYPunt(l,i),x0,y0,vx,vy,&x_final,&y_final);
		llCambiarPunt(l,i,x_final,y_final,0.0);

		i++;
	}
}

// ----------------------------------------------------------------
// Funcions que calculen el cami control (distancia, angle_control)
// ----------------------------------------------------------------

void calcular_angle_cami(void)
{
	float x_vect, y_vect, x0, y0, x0antic, y0antic, r;
	double a,b,c, discriminant;
	double z1,z2,z;
	float xf,yf;

	angle_cami_control = 0.0;	// Es posa a 0 per si no es pot calcular
	if (llNumPunts(llista)<2) return;

	// Calculem els vectors

	if (filtre_recta_activat) c_calcul_recta_regressio(llista,&x0,&y0,&x_vect,&y_vect,&r);
	else
	{
		c_calcul_recta_regressio(llista,&x0,&y0,&x_vect,&y_vect,&r_reg);
		c_calcul_r(llista,x0,y0,x_vect,y_vect,&r_reg);
	}

	x0antic = x0;
	y0antic = y0;

	// Calculem la recta a distncia adequada

	{
		float x_perp, y_perp;

		x_perp = y_vect;
		y_perp = -x_vect;

		if ((x0*x_perp+y0*y_perp)>0)
		{
			x_perp = -y_vect;
			y_perp = x_vect;
		}

		x0 = x0 + x_perp*distancia_a_mantenir;
		y0 = y0 + y_perp*distancia_a_mantenir;
	}

	// Calculem els parmetres de la equaci

	a = x_vect*x_vect + y_vect*y_vect;
	b = 2 * (x_vect*x0 + y_vect*y0);
	c = x0*x0 + y0*y0 - p_dist_radi_cami*p_dist_radi_cami;
	discriminant = b*b - 4*a*c;

	if (a<TOL_MIN) return;

	// Eliminem tots els punts del cami

	llEliminarTotsPunts(cami);

	// Comprovem tipus d'equaci

	if (discriminant>TOL_MIN)
	{
		z1 = (-b+sqrt(discriminant))/(2*a);
		z2 = (-b-sqrt(discriminant))/(2*a);

		xf = (float)(x0+z1*x_vect);
		yf = (float)(y0+z1*y_vect);
		llAfegirPunt(cami,xf,yf,0.0);

		xf = (float)(x0+z2*x_vect);
		yf = (float)(y0+z2*y_vect);
		llAfegirPunt(cami,xf,yf,0.0);
	}
	else if (discriminant<TOL_MIN)
	{
		z1 = (y_vect*x0 - x_vect*y0)/a;

		xf = (float)(z1*y_vect);
		yf = (float)(-z1*x_vect);

		llAfegirPunt(cami,xf,yf,0.0);
	}
	else
	{
		z = -b/(2*a);

		xf = (float)(x0+z*x_vect);
		yf = (float)(y0+z*y_vect);

		llAfegirPunt(cami,xf,yf,0.0);
	}

	// S'ha de calcular quin s el punt a seguir

	if (llNumPunts(cami)==1)
	{
		angle_cami_control = llAnglePunt(cami,0);
	}
	else if (llNumPunts(cami)==2)
	{
		float x1,y1,x2,y2,xr,yr,z;

		x1 = llObtenirXPunt(cami,0);
		y1 = llObtenirYPunt(cami,0);

		x2 = llObtenirXPunt(cami,1);
		y2 = llObtenirYPunt(cami,1);

		c_calcul_projeccio(0.0,0.0,x0antic,y0antic,x_vect,y_vect,&xr,&yr);

		z = (xr*y1)-(x1*yr);

		//sfSMessage("z = %f",z);

		if (seguir_dreta)
		{
			if (z>=0) angle_cami_control = llAnglePunt(cami,0);
			else angle_cami_control = llAnglePunt(cami,1);
		}
		else
		{
			if (z>=0) angle_cami_control = llAnglePunt(cami,1);
			else angle_cami_control = llAnglePunt(cami,0);
		}
	}
}

// -----------------------------------------------------------
// Funcions que calculen el control (distancia, angle_control)
// -----------------------------------------------------------

void c_calcul_control(float x, float y, float *angle, float *dist)
{
	float x_reg, y_reg, b0, b1;

	x_reg = x;
	y_reg = y;

	// Clcul de les variables de control

	if (modul(y_reg)>=modul(x_reg))
	{
		b1 = x_reg/(-y_reg);
		b0 = y_reg + (x_reg*x_reg/y_reg);
		c_angle_control_y(b0, b1, angle);
		c_distancia_control(x_reg,y_reg, dist);
	}
	else
	{
		b1 = -y_reg/x_reg;
		b0 = x_reg + (y_reg*y_reg/x_reg);
		c_angle_control_x(b0, b1,angle);
		c_distancia_control(x_reg,y_reg,dist);
	}
}

void c_angle_control_y(float b0, float b1, float *angle)
{
	if (seguir_dreta)
	{
		if ((b0<=0.0)&&(b1>=0.0)) (*angle) = (float)atan(b1);
		if ((b0<=0.0)&&(b1<=0.0)) (*angle) = (float)(-atan(-b1));
		if ((b0>=0.0)&&(b1<=0.0)) (*angle) = (float)(PI-atan(-b1));
		if ((b0>=0.0)&&(b1>=0.0)) (*angle) = (float)(-PI+atan(b1));
	}
	else
	{
		if ((b0>=0.0)&&(b1>=0.0)) (*angle) = (float)atan(b1);
		if ((b0>=0.0)&&(b1<=0.0)) (*angle) = (float)(-atan(-b1));
		if ((b0<=0.0)&&(b1<=0.0)) (*angle) = (float)(PI-atan(-b1));
		if ((b0<=0.0)&&(b1>=0.0)) (*angle) = (float)(-PI+atan(b1));
	}

	
	(*angle) = (float)((*angle) * 180.0 / PI);
}

void c_angle_control_x(float b0, float b1, float *angle)
{
	if (seguir_dreta)
	{
		if ((b0>=0.0)&&(b1>=0.0)) (*angle) = (float)(PI/2-atan(b1));
		if ((b0<=0.0)&&(b1<=0.0)) (*angle) = (float)(-PI/2+atan(-b1));
		if ((b0>=0.0)&&(b1<=0.0)) (*angle) = (float)(PI/2 + atan(-b1));
		if ((b0<=0.0)&&(b1>=0.0)) (*angle) = (float)(-PI/2 - atan(b1));
	}
	else
	{
		if ((b0<=0.0)&&(b1>=0.0)) (*angle) = (float)(PI/2-atan(b1));
		if ((b0>=0.0)&&(b1<=0.0)) (*angle) = (float)(-PI/2+atan(-b1));
		if ((b0<=0.0)&&(b1<=0.0)) (*angle) = (float)(PI/2 + atan(-b1));
		if ((b0>=0.0)&&(b1>=0.0)) (*angle) = (float)(-PI/2 - atan(b1));
	}

	(*angle) = (float)((*angle) * 180.0 / PI);
}

void c_distancia_control(float x, float y, float *dist)
{
	(*dist) = distancia(x,y);
}

float c_calcularXReg(void)
{
	if (llNumPunts(llista_emergencia)==0) return 0.0;
	else
	{
		float x,y;

		c_calcul_mitjanes(llista_emergencia,&x,&y);
		return x;
	}
}

float c_calcularYReg(void)
{
	if (llNumPunts(llista_emergencia)==0)
	{
		if (!seguir_dreta) return distancia_a_mantenir;
		else return -distancia_a_mantenir;
	}
	else
	{
		float x,y;

		c_calcul_mitjanes(llista_emergencia,&x,&y);
		return y;
	}
}

// --------------------
// Funcions del Colbert
// --------------------

void c_monitoritzar(int opcions)
{
	int i;
	static int interval = 0;

	interval++;
	if (interval!=10) return;	// Monitoritzem cada 10*100ms
	else interval = 0;

	if (opcions)
	{
		sfSMessage("");
		sfSMessage("MONITORITZACI");
		sfSMessage("--------------");
	}

	if (opcions&1)
	{
		sfSMessage("");
		sfSMessage("Variables de control:");
		sfSMessage("  Angle control: %f Distancia control: %f"
			, angle_control, distancia_control);
	}

	if (opcions&2)
	{	
		sfSMessage("");
		sfSMessage("Buffer correcte de dades:");
		llEscriureLlistaPunts(llista);
	}

	if (opcions&4)
	{	
		sfSMessage("");
		sfSMessage("Dades del sonar:");
		i = 1;
		while (i<=s_elements())
		{
			sfSMessage("  Punt %d: (%f , %f), dist: %f",i,s_x(i),s_y(i),
				distancia(s_x(i),s_y(i)));
			
			i++;
		}
	}

	if (opcions&8)
	{	
		sfSMessage("");
		sfSMessage("Valor r_reg: %f",r_reg);
	}
}

void c_visualitzar()
{
	if (visualitzacio&1) llEliminarTotsPunts(llista_emergencia);
	if (visualitzacio&2) llEliminarTotsPunts(llista);
	if (visualitzacio&4) llEliminarTotsPunts(cami);
}

// -----------------
// Funcions internes
// -----------------

// Funcions geomtriques: mdul (valor absolut) i distancia

float modul(float x)
{
	if (x<0.0) return -x;
	else return x;
}

float distancia(float x, float y)
{
	return (float)(sqrt(x*x+y*y));
}

// Funcions que realitzen una regresi (i clculs associats)

void c_calcul_mitjanes(LlistaPunts l, float *x_mitja, float *y_mitja)
{
	int i;

	(*x_mitja) = 0.0;
	(*y_mitja) = 0.0;

	if (llNumPunts(l)<1) return; // Comprovem que es pot fer una mitjana

	i = 0;
	while (i<llNumPunts(l))
	{
		(*x_mitja) += llObtenirXPunt(l,i);
		(*y_mitja) += llObtenirYPunt(l,i);

		i++;
	}

	(*x_mitja) /= llNumPunts(l);
	(*y_mitja) /= llNumPunts(l);
}

void c_calcul_variances(LlistaPunts l, float *cov_xy, float *sx_2, float *sy_2)
{
	float x_mitja, y_mitja;
	float resta_x, resta_y;
	int i;

	(*cov_xy) = 0.0;
	(*sx_2) = 0.0;
	(*sy_2) = 0.0;

	if(llNumPunts(l)<2) return; // Comprovem que es poden calcular variances

	c_calcul_mitjanes(l,&x_mitja,&y_mitja);

	i = 0;
	while (i<llNumPunts(l))
	{
		resta_x = llObtenirXPunt(l,i) - x_mitja;
		resta_y = llObtenirYPunt(l,i) - y_mitja;

		(*cov_xy) += resta_x * resta_y;
		(*sx_2) += resta_x * resta_x;
		(*sy_2) += resta_y * resta_y;

		i++;
	}

	(*cov_xy) /= llNumPunts(l)-1;
	(*sx_2) /= llNumPunts(l)-1;	
	(*sy_2) /= llNumPunts(l)-1;
}

void c_calcul_recta_regressio(LlistaPunts l, float *x0, float *y0, float *vx, float *vy, float *r)
{
	float cov_xy, sx_2, sy_2;
	float x_mitja, y_mitja;
	float vap_gran;
	float modul;

	assert(llNumPunts(l)>=2);

	c_calcul_variances(l,&cov_xy,&sx_2,&sy_2);
	c_calcul_mitjanes(l,&x_mitja,&y_mitja);

	vap_gran = (sx_2 + sy_2 + (float)sqrt(((sx_2-sy_2)*(sx_2-sy_2))+(4*(cov_xy*cov_xy))))/2;
	
	if (sx_2>sy_2)
	{
		if ((sx_2-vap_gran)>cov_xy)
		{
			(*vx) = -cov_xy/(sx_2-vap_gran);
			(*vy) = 1.0;
		}
		else
		{
			(*vx) = 1.0;
			(*vy) = -(sx_2-vap_gran)/cov_xy;
		}
	}
	else
	{
		if ((sy_2-vap_gran)>cov_xy)
		{
			(*vx) = 1.0;
			(*vy) = -cov_xy/(sy_2-vap_gran);
		}
		else
		{
			(*vx) = -(sy_2-vap_gran)/cov_xy;
			(*vy) = 1.0;
		}
	}

	modul =  distancia((*vx),(*vy));
	
	(*vx) = (*vx)/modul;
	(*vy) = (*vy)/modul;
	
	(*x0) = x_mitja;
	(*y0) = y_mitja;
	(*r) = (float)sqrt((cov_xy*cov_xy)/(sx_2*sy_2));
}

void c_calcul_projeccio(float x1, float y1, float x0, float y0, float vx, float vy, float *xp, float *yp)
{
	float t;

	// Es suposa que |(vx,vy)|=1

	t = (vx*(x1-x0)+vy*(y1-y0))/(vx*vx+vy*vy);
	(*xp) = x0 + t*vx;
	(*yp) = y0 + t*vy;
}

void c_calcul_r(LlistaPunts l, float x0, float y0, float vx, float vy, float *r)
{
	int i,j;
	float mitja,xp,yp,x,y,max;

	if (llNumPunts(l)<2)
	{
		(*r) = 0.0;
		return;
	}

	mitja = 0;

	i = 0;
	while (i<llNumPunts(l))
	{
		x = llObtenirXPunt(l,i);
		y = llObtenirYPunt(l,i);

		c_calcul_projeccio(x,y,x0,y0,vx,vy,&xp,&yp);
		mitja += distancia(xp-x,yp-y);

		i++;
	}

	mitja /= llNumPunts(l);
	
	max = 1.0; // Equivalent a max = 0.0;

	i = 0;
	while (i<llNumPunts(l))
	{
		j = 0;
		while (j<llNumPunts(l))
		{
			float dist;

			dist = distancia(llObtenirXPunt(l,i)-llObtenirXPunt(l,j),
				llObtenirYPunt(l,i)-llObtenirYPunt(l,j));

			if (dist>max) max = dist;

			j++;
		}

		i++;
	}

	// 5 Mtodes diferents de calcul r (per defecte 1)

	if (metode_calcul_r==1)
	{
		(*r) = 1 - (mitja*llNumPunts(l)/max);
	}
	else if (metode_calcul_r==2)
	{
		(*r) = 1 - (mitja/max);
	}
	else if (metode_calcul_r==3)
	{
		(*r) = 1 - (mitja*llNumPunts(l)/distancia_a_mantenir);
	}
	else if (metode_calcul_r==4)
	{
		(*r) = 1 - (mitja/distancia_a_mantenir);
	}
	else if (metode_calcul_r==5)
	{
		float x0,y0,vx,vy;
		c_calcul_recta_regressio(l,&x0,&y0,&vx,&vy,r);
	}
	else
	{
		(*r) = 1 - (mitja*llNumPunts(l)/max);
	}
}

// ----------------------------------------------
// Funcions de lectura i escriptura de parmetres
// ----------------------------------------------

float sfTGetDistanciaAMantenir(void)
{
	return distancia_a_mantenir;
}

void sfTSetDistanciaAMantenir(float d)
{
	if ((d>0.0)&&(d<4000.0)) distancia_a_mantenir = d;
	else sfSMessage("No es pot seguir una distancia de %f",d);
}

float sfTGetVelocitatAMantenir(void)
{
	return velocitat_a_mantenir;
}

void sfTSetVelocitatAMantenir(float v)
{
	if ((v<=400.0)&&(v>0.0)) velocitat_a_mantenir = v;
	else sfSMessage("No es pot seguir una velocitat de %f",v);
}

int sfTIsSeguimentDreta(void)
{
	return seguir_dreta;
}

void sfTSetSeguimentDreta(void)
{
	seguir_dreta = 1;
}

int sfTIsSeguimentEsquerra(void)
{
	return !seguir_dreta;
}

void sfTSetSeguimentEsquerra(void)
{
	seguir_dreta = 0;
}

int sfTGetMonitoritzacio(void)
{
	return monitoritzacio;
}

void sfTSetMonitoritzacio(int m)
{
	monitoritzacio = m;
}

int sfTGetVisualitzacio(void)
{
	return visualitzacio;
}

void sfTSetVisualitzacio(int v)
{
	visualitzacio = v;
}

float sfTGetAngleSensibilitat(void)
{
	return angle_sens;
}

void sfTSetAngleSensibilitat(float s)
{
	if ((s>=0)&&(s<=1)) angle_sens = s;
	else sfSMessage("La sensibilitat a l'angle ha d'estar entre 0 i 1");
}

float sfTGetDistanciaSensibilitat(void)
{
	return dist_sens;
}

void sfTSetDistanciaSensibilitat(float s)
{
	if ((s>=0)&&(s<=1)) dist_sens = s;
	else sfSMessage("La sensibilitat a la distancia ha d'estar entre 0 i 1");
}

float sfTGetParadaSensibilitat(void)
{
	return parada_sens;
}

void sfTSetParadaSensibilitat(float s)
{
	if ((s>=0)&&(s<=1)) parada_sens = s;
	else sfSMessage("La sensibilitat a la parada ha d'estar entre 0 i 1");
}

float sfTGetDistanciaRadiControl(void)
{
	return p_dist_radi_cami;
}

void sfTSetDistanciaRadiControl(float s)
{
	if ((s>0)&&(s<=10000.0)) p_dist_radi_cami = s;
	else sfSMessage("La Distancia del radi de control ha d'estar entre 0 i 10000.0");

}

float sfTGetDistanciaFocusControl(void)
{
	return p_focus_control;
}

void sfTSetDistanciaFocusControl(float d)
{
	if (d<=0.0)
	{
		sfSMessage("Distncia del focus de control ha de ser positiva");
		return;
	}

	p_focus_control = d;
}

int sfTIsFiltreAngleActivat(void)
{
	return filtre_angle_activat;
}

int sfTIsFiltreFocusActivat(void)
{
	return filtre_focus_activat;
}

int sfTIsFiltreRectaActivat(void)
{
	return filtre_recta_activat;
}

void sfTSetFiltreAngleActivat(void)
{
	filtre_angle_activat = 1;
}

void sfTSetFiltreFocusActivat(void)
{
	filtre_focus_activat = 1;
}

void sfTSetFiltreRectaActivat(void)
{
	filtre_recta_activat = 1;
}

void sfTSetFiltreAngleDesactivat(void)
{
	filtre_angle_activat = 0;
}

void sfTSetFiltreFocusDesactivat(void)
{
	filtre_focus_activat = 0;
}

void sfTSetFiltreRectaDesactivat(void)
{
	filtre_recta_activat = 0;
}

float sfTGetPAlfa1(void)
{
	return p_alfa_1;
}

void sfTSetPAlfa1(float p)
{
	p_alfa_1 = p;
}

float sfTGetPAlfa2(void)
{
	return p_alfa_2;
}

void sfTSetPAlfa2(float p)
{
	p_alfa_2 = p;
}

float sfTGetPMaxCorrecte(void)
{
	return p_max_correcte;
}

void sfTSetPMaxCorrecte(float p)
{
	if (p>1.0) p_max_correcte = p;
	else sfSMessage("PMaxCorrecte ha de ser major que 1");
}

float sfTGetPDistFocus(void)
{
	return p_dist_focus;
}

void sfTSetPDistFocus(float p)
{
	if (p>0.0) p_dist_focus = p;
	else sfSMessage("PDistFocus ha de tenir un valor positiu");
}

float sfTGetAngleControlEmergencia(void)
{
	return angle_control_emergencia;
}

float sfTGetRMinEmergencia(void)
{
	return r_min_emergencia;
}

float sfTGetRMaxEmergencia(void)
{
	return r_max_emergencia;
}

void sfTSetAngleControlEmergencia(float v)
{
	if (v<0.0)
	{
		sfSMessage("Angle emergencia ha de ser major que 0");
		return;
	}

	angle_control_emergencia = v;
}

void sfTSetREmergencia(float min, float max)
{
	if ((min>max)||(max>1.0))
	{
		sfSMessage("Restrccions de R Emergencia:");
		sfSMessage("  min<=max, max<=1.0");
		return;
	}

	r_min_emergencia = min;
	r_max_emergencia = max;
}

int sfTGetMetodeCalculR(void)
{
	return metode_calcul_r;
}

void sfTSetMetodeCalculR(int mr)
{
	if ((mr<1)||(mr>5))
	{
		sfSMessage("Mtode de calcul R incorrecte. Ha d'estar entre 1 i 5");
	}

	metode_calcul_r = mr;
}

float sfTGetConstDistEmergencia()
{
	return const_dist_emergencia;
}

void sfTSetConstDistEmergencia(float c)
{
	if (c<0)
	{
		sfSMessage("const_dist_emergencia ha de ser un valor positiu");
		return;
	}

	const_dist_emergencia = c;
}

// -------------
// Comportaments
// -------------

#include "beh.c"

// -----------------------------------------
// Fitxers de crrega i descrrega de la dll
// -----------------------------------------

EXPORT void
sfLoadInit(void)	// Fitxer que s'avalua al carregar-se la dll
{
	// Missatge d'inici

	sfSMessage("Iniciant crrega trackingObject.dll");

	// Crrega d'esquemes

	sfRegisterSchema("sfTracking",sfBEHAVIOR,sfTracking);

	// Crrega de funcions

	sfAddEvalFn("sfTGetDistanciaAMantenir", sfTGetDistanciaAMantenir, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetDistanciaAMantenir", sfTSetDistanciaAMantenir, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetVelocitatAMantenir", sfTGetVelocitatAMantenir, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetVelocitatAMantenir", sfTSetVelocitatAMantenir, sfVOID, 1, sfFLOAT);

	sfAddEvalFn("sfTIsSeguimentDreta", sfTIsSeguimentDreta, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTSetSeguimentDreta", sfTSetSeguimentDreta, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfTIsSeguimentEsquerra", sfTIsSeguimentEsquerra, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTSetSeguimentEsquerra", sfTSetSeguimentEsquerra, sfVOID, 0, sfVOID);

	sfAddEvalFn("sfTGetMonitoritzacio", sfTGetMonitoritzacio, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTSetMonitoritzacio", sfTSetMonitoritzacio, sfVOID, 1, sfINT);
	sfAddEvalFn("sfTGetVisualitzacio", sfTGetVisualitzacio, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTSetVisualitzacio", sfTSetVisualitzacio, sfVOID, 1, sfINT);

	sfAddEvalFn("sfTGetAngleSensibilitat", sfTGetAngleSensibilitat, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetAngleSensibilitat", sfTSetAngleSensibilitat, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetDistanciaSensibilitat", sfTGetDistanciaSensibilitat, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetDistanciaSensibilitat", sfTSetDistanciaSensibilitat, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetParadaSensibilitat", sfTGetParadaSensibilitat, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetParadaSensibilitat", sfTSetParadaSensibilitat, sfVOID, 1, sfFLOAT);

	sfAddEvalFn("sfTGetDistanciaRadiControl", sfTGetDistanciaRadiControl, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetDistanciaRadiControl", sfTSetDistanciaRadiControl, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetDistanciaFocusControl", sfTGetDistanciaFocusControl, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetDistanciaFocusControl", sfTSetDistanciaFocusControl, sfVOID, 1, sfFLOAT);

	sfAddEvalFn("sfTIsFiltreAngleActivat", sfTIsFiltreAngleActivat, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTIsFiltreFocusActivat", sfTIsFiltreFocusActivat, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTIsFiltreRectaActivat", sfTIsFiltreRectaActivat, sfINT, 0, sfVOID);

	sfAddEvalFn("sfTSetFiltreAngleActivat", sfTSetFiltreAngleActivat, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfTSetFiltreFocusActivat", sfTSetFiltreFocusActivat, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfTSetFiltreRectaActivat", sfTSetFiltreRectaActivat, sfVOID, 0, sfVOID);

	sfAddEvalFn("sfTSetFiltreAngleDesactivat", sfTSetFiltreAngleDesactivat, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfTSetFiltreFocusDesactivat", sfTSetFiltreFocusDesactivat, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfTSetFiltreRectaDesactivat", sfTSetFiltreRectaDesactivat, sfVOID, 0, sfVOID);

	sfAddEvalFn("sfTGetPAlfa1", sfTGetPAlfa1, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetPAlfa1", sfTSetPAlfa1, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetPAlfa2", sfTGetPAlfa2, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetPAlfa2", sfTSetPAlfa2, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetPMaxCorrecte", sfTGetPMaxCorrecte, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetPMaxCorrecte", sfTSetPMaxCorrecte, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTGetPDistFocus", sfTGetPDistFocus, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetPDistFocus", sfTSetPDistFocus, sfVOID, 1, sfFLOAT);

	sfAddEvalFn("sfTGetAngleControlEmergencia", sfTGetAngleControlEmergencia, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTGetRMinEmergencia", sfTGetRMinEmergencia, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTGetRMaxEmergencia", sfTGetRMaxEmergencia, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetAngleControlEmergencia", sfTSetAngleControlEmergencia, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfTSetREmergencia", sfTSetREmergencia, sfVOID, 2, sfFLOAT, sfFLOAT);

	sfAddEvalFn("sfTGetMetodeCalculR", sfTGetMetodeCalculR, sfINT, 0, sfVOID);
	sfAddEvalFn("sfTSetMetodeCalculR", sfTSetMetodeCalculR, sfVOID, 1, sfINT);

	sfAddEvalFn("sfTGetConstDistEmergencia", sfTGetConstDistEmergencia, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfTSetConstDistEmergencia", sfTSetConstDistEmergencia, sfVOID, 1, sfFLOAT);
	
	// Inicialitzaci de variables

	llista = llCrearLlistaPunts();
	cami = llCrearLlistaPunts();
	llista_emergencia = llCrearLlistaPunts();

	// Missatge final

	sfSMessage("Crrega de trackingObject.dll finalitzada");
}

EXPORT void
sfLoadExit(void)	// Fitxer que s'avalua al descarregar-se la dll
{
	// Descrrega de variables

	llDestruirLlistaPunts(llista);
	llDestruirLlistaPunts(cami);
	llDestruirLlistaPunts(llista_emergencia);

	// Missatge de descrrega

	sfSMessage("Descrrega de trackingObject.dll efectuada");
}
