// -------------------
// Inclusi de fitxers
// -------------------

#include "sfUtils.h"
#include <assert.h>

// ------------------------------
// Definici de funcions internes
// ------------------------------

float CalcularNik(float u, Vector knots, int i, int k);
float CalcularBkn(int k, int n, float u);
float CalcularCnk(int n, int k);
float factorial(int n);
float potencia(float u, int k);

// ------------------------------
// Implementaci del tipus Vector
// ------------------------------

EXPORT Vector CrearVector(int n)
{
	Vector v;

	assert(n > 0);
	v.dim = n;
	v.telem = (float *)malloc(n * sizeof (float));

	assert(v.telem != NULL); 
	return v;
}

EXPORT void InicialitzarVector(Vector v)
{
	int i;

	i = 0;
	while (i<DimVector(v))
	{
		AssigVector(v,i,0.0);

		i++;
	}
}

EXPORT int DimVector(Vector v)
{
	return v.dim;
}

EXPORT void AssigVector(Vector v, int i, float f)
{
	assert(0 <= i && i < v.dim);
	v.telem[i] = f;
}

EXPORT float ConsVector(Vector v, int i)
{
	assert(0 <= i && i < v.dim);
	return v.telem[i];
}

EXPORT void DestruirVector(Vector v)
{
	free(v.telem);
}

EXPORT void EscriureVector(Vector v)
{
	int i;

	i = 0;
	while (i<DimVector(v))
	{
		sfSMessage("  v[%d] = %f", i, ConsVector(v,i));

		i++;
	}
}

EXPORT LlistaPunts ObtenirLlistaVectors(Vector x, Vector y, Vector angle)
{
	LlistaPunts llista;
	int i;

	llista = llCrearLlistaPunts();

	if (DimVector(x)!=DimVector(y) || DimVector(y)!=DimVector(angle))
	{
		sfSMessage("Els vectors han de ser del mateix tamany");
		sfSMessage("Es retorna una llista buida");
		return llista;
	}

	i = 0;
	while (i<DimVector(x))
	{
		llAfegirPunt(llista,ConsVector(x,i),ConsVector(y,i),ConsVector(angle,i));

		i++;
	}

	return llista;
}

EXPORT LlistaPunts ObtenirLlistaVectorsGlobals(Vector x, Vector y, Vector angle)
{
	LlistaPunts llista;
	int i;

	llista = llCrearLlistaPunts();

	if (DimVector(x)!=DimVector(y) || DimVector(y)!=DimVector(angle))
	{
		sfSMessage("Els vectors han de ser del mateix tamany");
		sfSMessage("Es retorna una llista buida");
		return llista;
	}

	i = 0;
	while (i<DimVector(x))
	{
		llAfegirPuntGlobal(llista,ConsVector(x,i),ConsVector(y,i),ConsVector(angle,i));

		i++;
	}

	return llista;
}

// -----------------------------------
// Implementaci del tipus LlistaPunts
// -----------------------------------

telemento *NuevoElemento(void)
{
	telemento *q = ((telemento *)malloc(sizeof(telemento)));
	if (!q)
	{
		perror("Insuficiente memoria");
		exit(1);
	}
	return q;
}


EXPORT LlistaPunts llCrearLlistaPunts(void)
{
	LlistaPunts l;

	l.punts = (telemento **)malloc(sizeof(telemento *));
	l.punts[0] = NuevoElemento();
	l.punts[0]->siguiente = NULL;

	l.numpunts = (int *)malloc(sizeof(int));
	l.numpunts[0] = 0;

	return l;
}

EXPORT void llDestruirLlistaPunts(LlistaPunts l)
{
	llEliminarTotsPunts(l);
	free(l.numpunts);
	free(l.punts);
}

EXPORT int llNumPunts(LlistaPunts l)
{
	return l.numpunts[0];
}

EXPORT void llAfegirPunt(LlistaPunts l, float x, float y, float angle)
{
	point *aux;
	telemento *q;

	aux = sfCreateLocalPoint(x,y,angle);

	q = NuevoElemento();
	q->punt = aux;
	sfAddPoint(q->punt);

	q->siguiente = l.punts[0];
	l.punts[0] = q;

	l.numpunts[0]++;
}

EXPORT void llAfegirPuntGlobal(LlistaPunts l, float x, float y, float angle)
{
	point *aux;
	telemento *q;

	aux = sfCreateGlobalPoint(x,y,angle);

	q = NuevoElemento();
	q->punt = aux;
	sfAddPoint(q->punt);

	q->siguiente = l.punts[0];
	l.punts[0] = q;

	l.numpunts[0]++;
}

EXPORT void llInsertarPunt(LlistaPunts l, float x, float y, float angle, int pos)
{
	int i;
	telemento *q, *r;
	point *aux;

	// Es pot insertar en posici de llNumPunts(l)

	if (pos<0 || pos>llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte. No es realitza cap operaci");
		return;
	}

	if (pos==llNumPunts(l))
	{
		llAfegirPunt(l,x,y,angle); 
		return;
	}

	aux = sfCreateLocalPoint(x,y,angle);	
	q = NuevoElemento();
	q->punt = aux;
	sfAddPoint(q->punt);
	
	i = llNumPunts(l) - 1;
	r = l.punts[0];
	while (i>pos)
	{
		r = r->siguiente;
		i--;
	}

	q->siguiente = r->siguiente;
	r->siguiente = q;

	l.numpunts[0]++;
}

EXPORT void llInsertarPuntGlobal(LlistaPunts l, float x, float y, float angle, int pos)
{
	int i;
	telemento *q, *r;
	point *aux;

	// Es pot insertar en posici de llNumPunts(l)

	if (pos<0 || pos>llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte. No es realitza cap operaci");
		return;
	}

	if (pos==llNumPunts(l))
	{
		llAfegirPuntGlobal(l,x,y,angle); 
		return;
	}

	aux = sfCreateGlobalPoint(x,y,angle);	
	q = NuevoElemento();
	q->punt = aux;
	sfAddPoint(q->punt);
	
	i = llNumPunts(l) - 1;
	r = l.punts[0];
	while (i>pos)
	{
		r = r->siguiente;
		i--;
	}

	q->siguiente = r->siguiente;
	r->siguiente = q;

	l.numpunts[0]++;
}

EXPORT void llEliminarPunt(LlistaPunts l, int pos)
{
	telemento *q, *aux;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es realitza cap operaci");
		return;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte. No es realitza cap operaci");
		return;
	}

	if(pos==llNumPunts(l)-1)
	{
		llEliminarUltimPunt(l);
		return;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos+1)
	{
		q = q->siguiente;
		i--;
	}

	aux = q->siguiente;
	q->siguiente = aux->siguiente;
	sfRemPoint(aux->punt);
	free(aux);
	l.numpunts[0]--;
}

EXPORT void llEliminarPrimerPunt(LlistaPunts l)
{
	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es realitza cap operaci");
		return;
	}

	llEliminarPunt(l,0);
}

EXPORT void llEliminarUltimPunt(LlistaPunts l)
{
	telemento *q;

	if (llNumPunts(l)==0) return;

	q = l.punts[0];
	l.punts[0] = l.punts[0]->siguiente;

	sfRemPoint(q->punt);
	free(q);

	l.numpunts[0]--;
}

EXPORT void llEliminarTotsPunts(LlistaPunts l)
{
	while(llNumPunts(l)>0)
	{
		llEliminarUltimPunt(l);
	}
}

EXPORT void llCambiarPunt(LlistaPunts l, int pos, float x, float y, float angle)
{
	telemento *q;
	int i;
	point *p;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot cambiar el punt");
		return;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	p = q->punt;
	p->x = x;
	p->y = y;
	p->th = angle;
	sfSetGlobalCoords(p);
}

EXPORT void llCambiarPuntGlobal(LlistaPunts l, int pos, float x, float y, float angle)
{
	telemento *q;
	int i;
	point *p;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot cambiar el punt");
		return;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	p = q->punt;
	p->ax = x;
	p->ay = y;
	p->ath = angle;
	sfSetLocalCoords(p);
}

EXPORT float llObtenirXPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->x;
}

EXPORT float llObtenirYPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->y;
}

EXPORT float llObtenirAnglePunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->th;
}

EXPORT float llObtenirXGlobalPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->ax;
}

EXPORT float llObtenirYGlobalPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->ay;
}

EXPORT float llObtenirAngleGlobalPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return 0.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return q->punt->ath;
}

EXPORT Vector llObtenirVectorXLlista(LlistaPunts l)
{
	Vector v;
	int i;

	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirXPunt(l,i));

		i++;
	}

	return v;
}

EXPORT Vector llObtenirVectorYLlista(LlistaPunts l)
{
	Vector v;
	int i;
	
	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirYPunt(l,i));

		i++;
	}

	return v;
}

EXPORT Vector llObtenirVectorAngleLlista(LlistaPunts l)
{
	Vector v;
	int i;
	
	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirAnglePunt(l,i));

		i++;
	}

	return v;
}

EXPORT Vector llObtenirVectorXGlobalLlista(LlistaPunts l)
{
	Vector v;
	int i;

	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirXGlobalPunt(l,i));

		i++;
	}

	return v;
}

EXPORT Vector llObtenirVectorYGlobalLlista(LlistaPunts l)
{
	Vector v;
	int i;

	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirYGlobalPunt(l,i));

		i++;
	}

	return v;
}

EXPORT Vector llObtenirVectorAngleGlobalLlista(LlistaPunts l)
{
	Vector v;
	int i;

	if (llNumPunts(l)==0)
	{
		v = CrearVector(1);
		InicialitzarVector(v);
		sfSMessage("Llista buida. Es crea un vector de 1 element. Valor 0");

		return v;
	}

	v = CrearVector(llNumPunts(l));

	i = 0;
	while (i<llNumPunts(l))
	{
		AssigVector(v,i,llObtenirAngleGlobalPunt(l,i));

		i++;
	}

	return v;
}

EXPORT void llEscriureLlistaPunts(LlistaPunts l)
{
	int i;

	i = 0;
	while (i<llNumPunts(l))
	{
		sfSMessage("Punt %d: ( %f , %f , <%f )", i,
			llObtenirXPunt(l,i), llObtenirYPunt(l,i),
			llObtenirAnglePunt(l,i));

		i++;
	}
}

EXPORT void llEscriureLlistaPuntsGlobals(LlistaPunts l)
{
	int i;

	i = 0;
	while (i<llNumPunts(l))
	{
		sfSMessage("Punt %d: ( %f , %f , <%f )", i,
			llObtenirXGlobalPunt(l,i), llObtenirYGlobalPunt(l,i),
			llObtenirAngleGlobalPunt(l,i));

		i++;
	}
}

// -----------------------------------
// Implementaci Funcions geomtriques
// -----------------------------------

EXPORT LlistaPunts CalcularBezier(LlistaPunts l, int k, int num_punts)
{
	LlistaPunts llista;
	Vector x,y;
	int j,n;
	float interval;

	llista = llCrearLlistaPunts();
	x = llObtenirVectorXLlista(l);
	y = llObtenirVectorYLlista(l);

	n = DimVector(x) - 1;
	if (n<1)
	{
			sfSMessage("Llista de punts incorrecte");
			sfSMessage("Es retorna llista buida");
			return llista;
	}

	num_punts--;
	if (num_punts<1)
	{
		sfSMessage("Nombre de punts incorrecte");
		sfSMessage("Es retorna llista buida");
		return llista;
	}

	interval = (float)(1.0/num_punts);

	j = 0;

	while (j<=num_punts)
	{
		float u;
		float x_bezier, y_bezier, bkn;

		u = j*interval;

		// Calcul de valors de x i de y de bezier

		x_bezier = 0.0;
		y_bezier = 0.0;

		k = 0;
		while (k<=n)
		{
			bkn = CalcularBkn(k,n,u);
			x_bezier += ConsVector(x,k)*bkn;
			y_bezier += ConsVector(y,k)*bkn;

			k++;
		}

		// Fi calcul
		llAfegirPunt(llista,x_bezier,y_bezier,0.0);
		
		j++;
	}

	DestruirVector(x);
	DestruirVector(y);

	return llista;
}

EXPORT LlistaPunts CalcularSpline(LlistaPunts l, Vector knots, int k, int num_punts)
{
	LlistaPunts llista;
	Vector x,y;
	int n,i,j;
	float uinici, ufi, interval;


	x = llObtenirVectorXLlista(l);
	y = llObtenirVectorYLlista(l);

	llista = llCrearLlistaPunts();
	
	n = DimVector(x) - 1;
	if (n<1)
	{
			sfSMessage("Llista de punts incorrecte");
			sfSMessage("Es retorna llista buida");
			return llista;
	}

	if (DimVector(knots)!=n+k+1)
	{
			sfSMessage("Vector de nusos incorrecte");
			sfSMessage("Es retorna llista buida");
			return llista;
	}

	if (k<1)
	{
			sfSMessage("Vector de k incorrecte");
			sfSMessage("Es retorna llista buida");
			return llista;
	}

	uinici = ConsVector(knots,k-1);
	ufi = ConsVector(knots,n+1);

	// Falta comprovar vector de knots correcte

	num_punts--;
	if (num_punts<1)
	{
		sfSMessage("Nombre de punts incorrecte");
		sfSMessage("Es retorna llista buida");
		return llista;
	}

	interval = (ufi-uinici)/num_punts;

	j = 0;
	
	while (j<=num_punts)
	{
		float u;
		float x_spline, y_spline, nik;

		u = uinici+(j*interval);

		// Calcul de valors de x i de y de spline

		x_spline = 0.0;
		y_spline = 0.0;

		i = 0;
		while (i<=n)
		{
			nik = CalcularNik(u,knots,i,k);
			x_spline += ConsVector(x,i)*nik;
			y_spline += ConsVector(y,i)*nik;

			i++;
		}

		// Fi calcul

		llAfegirPunt(llista,x_spline,y_spline,0.0);
		
		j++;
	}

	DestruirVector(x);
	DestruirVector(y);

	return llista;
}

EXPORT LlistaPunts CalcularStandardSpline(LlistaPunts l, int k, int num_punts)
{
	LlistaPunts llista;
	Vector knots;
	int i,n;

	llista = llCrearLlistaPunts();
	n = llNumPunts(l)-1;
	knots = CrearVector(n+k+1);

	i = 0;
	while (i<DimVector(knots))
	{
		AssigVector(knots,i,(float)i);

		i++;
	}

	llista = CalcularSpline(l,knots,k,num_punts);
	DestruirVector(knots);

	return llista;
}

EXPORT int MinimaDistanciaLlistaPunts(LlistaPunts l)
{
	//Es fan clculs amb distncies^2 per augmentar la rapidesa de clculs

	int i, min;
	float x, y;
	double distmin;
	
	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista buida. Retorn negatiu");
		return -1;
	}

	x = llObtenirXPunt(l,0);
	y = llObtenirYPunt(l,0);
	distmin = pow(x,2)+pow(y,2);

	min = 0;
	i = 1;
	while (i<llNumPunts(l))
	{
		x = llObtenirXPunt(l,i);
		y = llObtenirYPunt(l,i);
		if ((pow(x,2)+pow(y,2))<distmin)
		{
			distmin = pow(x,2)+pow(y,2);
			min = i;
		}

		i++;
	}

	return min;
}

EXPORT int MaximaDistanciaLlistaPunts(LlistaPunts l)
{
	//Es fan clculs amb distncies^2 per augmentar la rapidesa de clculs

	int i, max;
	float x, y;
	double distmax;
	
	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista buida. Retorn negatiu");
		return -1;
	}

	x = llObtenirXPunt(l,0);
	y = llObtenirYPunt(l,0);
	distmax = pow(x,2)+pow(y,2);

	max = 0;
	i = 1;
	while (i<llNumPunts(l))
	{
		x = llObtenirXPunt(l,i);
		y = llObtenirYPunt(l,i);
		if ((pow(x,2)+pow(y,2))>distmax)
		{
			distmax = pow(x,2)+pow(y,2);
			max = i;
		}

		i++;
	}

	return max;
}

EXPORT float llDistanciaPunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return -1.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return sfPointDist(q->punt);
}

EXPORT float llAnglePunt(LlistaPunts l, int pos)
{
	telemento *q;
	int i;

	if (llNumPunts(l)==0)
	{
		sfSMessage("Llista Buida. No es pot obtenir punt");
		return 0.0;
	}

	if (pos<0 || pos>=llNumPunts(l))
	{
		sfSMessage("Index pos incorrecte");
		return -1.0;
	}

	q = l.punts[0];
	i = llNumPunts(l)-1;

	while (i>pos)
	{
		q = q->siguiente;
		i--;
	}

	return sfPointPhi(q->punt);
}

EXPORT float llLongitudLlista(LlistaPunts l)
{
	return llLongitudLlistaInterval(l,0,llNumPunts(l)-1);
}

EXPORT float llLongitudLlistaInterval(LlistaPunts l, int inici, int fi)
{
	int i;
	float longitud, x, y;

	if (inici<=0)
	{
		inici = 0;
	}
	if (inici>=llNumPunts(l)-1) return 0.0;
	if (fi<0) fi = llNumPunts(l);
	if (fi<=inici) return 0.0;
	if (fi>=llNumPunts(l)) fi = llNumPunts(l)-1;

	longitud = 0.0;

	i = inici;
	while (i<=(fi-1))
	{
		x = llObtenirXPunt(l,i) - llObtenirXPunt(l,i+1);
		y = llObtenirYPunt(l,i) - llObtenirYPunt(l,i+1);

		longitud += (float)sqrt(pow(x,2)+pow(y,2));

		i++;
	}

	return longitud;
}

// ----------------------------------
// Implementaci de Funcions internes
// ----------------------------------

float CalcularNik(float u, Vector knots, int i, int k)
{
	float num1, num2, den1, den2, valorfinal;

	valorfinal = 0.0;

	if (k==1)
	{
		if ((ConsVector(knots,i)<=u)&&(u<ConsVector(knots,i+1)))
			valorfinal = 1.0;
		else 
			valorfinal = 0.0;
	}
	else
	{
		num1 = u - ConsVector(knots,i);
		den1 = ConsVector(knots,i+k-1) - ConsVector(knots,i);

		num2 = ConsVector(knots, i+k) -u;
		den2 = ConsVector(knots,i+k) - ConsVector(knots,i+1);

		if (den1!=0.0)
		{
			valorfinal += (num1/den1)*CalcularNik(u,knots,i,k-1);
		}

		if (den2!=0.0)
		{
			valorfinal += (num2/den2)*CalcularNik(u,knots,i+1,k-1);
		}
	}

	return valorfinal;
}

float CalcularBkn(int k, int n, float u)
{
	float aux;

	aux = CalcularCnk(n,k)*potencia(u,k)*potencia(1-u,n-k);
	return aux;
}

float CalcularCnk(int n, int k)
{
	float aux;
	aux = factorial(n);
	aux /= factorial(k);
	aux /= factorial(n-k);
	return aux;
}

float factorial(int n)
{
	float aux;

	assert(n>=0);
	aux = 1.0;
	while (n>0)
	{
		aux *= n;
		n--;
	}

	return aux;
}

float potencia(float u, int k)
{
	return (float)pow((double)u,(double)k);
}


// Funcio de proves
EXPORT void
sfLoadInit(void)	// Fitxer que s'avalua al carregar-se la dll
{
	// Proves

	LlistaPunts llista;
	float aux;

	llista = llCrearLlistaPunts();

	llAfegirPunt(llista,0.0,0.0,0.0);
	llAfegirPunt(llista,100.0,0.0,0.0);
	llAfegirPunt(llista,400.0,400.0,0.0);
	llAfegirPunt(llista,600.0,400.0,0.0);

	aux = llLongitudLlista(llista);
	sfSMessage("Longitud total: %f",aux);
	sfSMessage("");

	aux = llLongitudLlistaInterval(llista,0,1);
	sfSMessage("Interval 0,1: %f",aux);

	aux = llLongitudLlistaInterval(llista,0,2);
	sfSMessage("Interval 0,2: %f",aux);

	aux = llLongitudLlistaInterval(llista,0,3);
	sfSMessage("Interval 0,3: %f",aux);

	aux = llLongitudLlistaInterval(llista,1,2);
	sfSMessage("Interval 1,2: %f",aux);

	aux = llLongitudLlistaInterval(llista,1,3);
	sfSMessage("Interval 1,3: %f",aux);

	aux = llLongitudLlistaInterval(llista,2,3);
	sfSMessage("Interval 2,3: %f",aux);

	sfSMessage("");

	aux = llLongitudLlistaInterval(llista,0,0);
	sfSMessage("Interval 0,0: %f",aux);

	aux = llLongitudLlistaInterval(llista,1,1);
	sfSMessage("Interval 1,1: %f",aux);
	
	aux = llLongitudLlistaInterval(llista,2,2);
	sfSMessage("Interval 2,2: %f",aux);

	aux = llLongitudLlistaInterval(llista,3,3);
	sfSMessage("Interval 3,3: %f",aux);

	llDestruirLlistaPunts(llista);

	// Missatge inici i final
	sfSMessage("Iniciant crrega sfUtils.dll");
	sfSMessage("Crrega de sfUtils.dll finalitzada");
}
